var obj = {
  name: 'ATRBands',
  summary: 'Average True Range (ATR) Bands',
  description: 'Calculated on the selected price source.'
}

obj.info = function () {
  obj.overlay = true

  obj._upperCurve = {
    title: 'Upper',
    color: 'indianred'
  }

  obj._lowerCurve = {
    title: 'Lower',
    color: 'indianred'
  }

  obj.curves = [obj._upperCurve, obj._lowerCurve]

  obj._atrPeriods = {
    name: 'ATR periods',
    defaultValue: 14
  }

  obj._atrMultiplier = {
    name: 'ATR multiplier',
    defaultValue: 1.0,
    type: Spock.ParameterType.Double
  }

  obj._shift = {
    name: 'Shift',
    defaultValue: 0
  }

  obj.parameters = [obj._atrPeriods, obj._atrMultiplier, obj._shift]
  obj.requiredSource = Spock.RequiredSource.Bar
}

obj._preExec = function () {
  obj._upperCurve.barsOffset = obj._shift.value
  obj._lowerCurve.barsOffset = obj._shift.value
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._preExec()
    obj._atr = new Array(obj._atrPeriods.value)
  }

  // calculate True Range
  var curHigh = Plot.high(period)
  var curLow = Plot.low(period)
  if (period === 0) {
    obj._atr[0] = curHigh - curLow
  } else {
    var prevClose = Plot.close(period - 1)

    var method1 = curHigh - curLow
    var method2 = curHigh - prevClose
    if (method2 < 0) {
      method2 = -method2
    }
    var method3 = curLow - prevClose
    if (method3 < 0) {
      method3 = -method3
    }

    var curTr = Math.max(method1, method2, method3)
    if (period >= obj._atrPeriods.value) {
      obj._atr[period] =
        ((obj._atr[period - 1] * (obj._atrPeriods.value - 1)) + curTr) / obj._atrPeriods.value
    } else if (period < obj._atrPeriods.value - 1) {
      obj._atr[period] = curTr
    } else {
      obj._atr[period] = curTr
      var trs = obj._atr.slice(0, obj._atrPeriods.value + 1)
      var sum_ = Plot.sum(trs, trs.length)
      obj._atr[period] = sum_ / obj._atrPeriods.value
    }
  }

  if (period < obj._atrPeriods.value) {
    return
  }

  var price = Plot.getTick(period)
  var offset = obj._atr[period] * obj._atrMultiplier.value
  return [price + offset, price - offset]
}
