var obj = {
  name: 'BW MFI',
  summary: 'Market Facilitation Index',
  description: 'Colors: green bar = green, fade bar = red, fake bar = gray, squat bar = blue'
}

obj.info = function () {
  obj._mfiCurve = {
    title: 'MFI',
    style: Spock.LineCurveStyle.Columns,
    lineWidth: 1,
    hasPerBarColor: true
  }
  obj.curves = [obj._mfiCurve]

  // Colors from http://traders.com/documentation/feedbk_docs/1997/02/0297tradetips.html
  // swapped red and blue since squat bars are blue,
  // plus red (fade) is opposite to green

  obj._greenColor = {
    name: 'Green color',
    defaultValue: 'forestgreen',
    type: Spock.ParameterType.Color,
  }

  obj._fadeColor = {
    name: 'Fade color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }

  obj._fakeColor = {
    name: 'Fake color',
    defaultValue: 'darkgray',
    type: Spock.ParameterType.Color,
  }

  obj._squatColor = {
    name: 'Squat color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }

  obj.parameters = [
    obj._greenColor,
    obj._fadeColor,
    obj._fakeColor,
    obj._squatColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Bar
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._mfi = new Array(1)
  }

  var vol = Plot.volume(period)
  if (vol <= 0) {
    return
  }

  var mfi = (Plot.high(period) - Plot.low(period)) / vol
  obj._mfi[period] = mfi

  if (period < 1) {
    return
  }

  var prevMfi = obj._mfi[period - 1]
  if (prevMfi == null) {
    return
  }

  var mfiUp = mfi > prevMfi
  var mfiDown = !mfiUp && mfi < prevMfi

  var prevVol = Plot.volume(period - 1)
  var volUp = vol > prevVol
  var volDown = !volUp && vol < prevVol

  var color
  if (volUp) {
    if (mfiUp) {
      color = obj._greenColor.value
    } else if (mfiDown) {
      color = obj._squatColor.value
    }
  } else if (volDown) {
    if (mfiUp) {
      color = obj._fakeColor.value
    } else if (mfiDown) {
      color = obj._fadeColor.value
    }
  }

  return [{ value: mfi, color: color, hollow: color === undefined }]
}
