var obj = {
  name: 'CAEMA',
  summary: 'Crossover Arrow EMA',
  description: 'Plots an arrow based on an EMA crossover, ' +
    'the arrow points to the first candle that opened after the cross.',
  group: 'Other',
}

obj.info = function () {
  obj.overlay = true
  obj.curves = [{
    title: 'Crossover',
    style: Spock.LineCurveStyle.Arrows,
    hasPerBarColor: true
  }]

  obj._slowPeriods = {
    name: 'Slow EMA periods',
    defaultValue: 55
  }

  obj._fastPeriods = {
    name: 'Fast EMA periods',
    defaultValue: 21
  }

  obj._overColor = {
    name: 'Over color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }
  obj._underColor = {
    name: 'Under color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }

  obj.parameters = [
    obj._slowPeriods,
    obj._fastPeriods,
    obj._overColor,
    obj._underColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Bar
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._slow = new Array(obj._slowPeriods.value)
    obj._fast = new Array(obj._fastPeriods.value)
    obj._isSlowUnder = new Array(obj._slowPeriods.value)
  }

  if (period === obj._slowPeriods.value) {
    obj._slow[period] = Plot.sma(Plot.getTicks(period, obj._slowPeriods.value), obj._slowPeriods.value)
  } else if (period > obj._slowPeriods.value) {
    obj._slow[period] = Plot.ema(Plot.getTick(period), obj._slowPeriods.value, obj._slow[period - 1])
  }

  if (period === obj._fastPeriods.value) {
    obj._fast[period] = Plot.sma(Plot.getTicks(period, obj._fastPeriods.value), obj._fastPeriods.value)
  } else if (period > obj._fastPeriods.value) {
    obj._fast[period] = Plot.ema(Plot.getTick(period), obj._fastPeriods.value, obj._fast[period - 1])
  }

  obj._isSlowUnder[period] = obj._slow[period] <= obj._fast[period]

  if (obj._isSlowUnder[period - 2] == undefined) {
    return
  }

  var didChange = obj._isSlowUnder[period - 1] != obj._isSlowUnder[period - 2]
  if (!didChange) {
    return
  }

  if (obj._isSlowUnder[period - 1]) {
    return [{
      value: Plot.low(period),
      color: obj._overColor.value,
      hollow: true
    }]
  }
  return [{
    value: Plot.high(period),
    color: obj._underColor.value,
    hollow: false
  }]
}
