var obj = {
  name: 'CASMA',
  summary: 'Crossover Arrow SMA',
  description: 'Plots an arrow based on an SMA crossover, ' +
    'the arrow points to the first candle that opened after the cross.',
  group: 'Other',
}

obj.info = function () {
  obj.overlay = true
  obj.curves = [{
    title: 'Crossover',
    style: Spock.LineCurveStyle.Arrows,
    hasPerBarColor: true
  }]

  obj._slowPeriods = {
    name: 'Slow SMA periods',
    defaultValue: 55
  }

  obj._fastPeriods = {
    name: 'Fast SMA periods',
    defaultValue: 21
  }

  obj._overColor = {
    name: 'Over color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }
  obj._underColor = {
    name: 'Under color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }

  obj.parameters = [
    obj._slowPeriods,
    obj._fastPeriods,
    obj._overColor,
    obj._underColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Bar
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._isSlowUnder = new Array(obj._slowPeriods.value)
  }

  // assert period1 > period2
  var prices = Plot.getTicks(period, obj._slowPeriods.value)
  if (prices == null) {
    return
  }

  var slowMa = Plot.sma(prices, obj._slowPeriods.value)
  var fastMa = Plot.sma(prices, obj._fastPeriods.value)

  obj._isSlowUnder[period] = slowMa <= fastMa

  if (obj._isSlowUnder[period - 2] == undefined) {
    return
  }

  var didChange = obj._isSlowUnder[period - 1] != obj._isSlowUnder[period - 2]
  if (!didChange) {
    return
  }

  if (obj._isSlowUnder[period - 1]) {
    return [{
      value: Plot.low(period),
      color: obj._overColor.value,
      hollow: true
    }]
  }
  return [{
    value: Plot.high(period),
    color: obj._underColor.value,
    hollow: false
  }]
}
