var obj = {
  name: 'CCSMA',
  summary: 'Crossover Candle SMA',
  description: 'Color candlesticks based on SMA crossover',
  group: 'Other',
}

obj.info = function () {
  obj.overlay = true
  obj.curves = [{
    title: 'CCSMA',
    type: Spock.CurveType.Bar,
    style: Spock.BarCurveStyle.Candle,
    hasPerBarColor: true,
    zIndex: 1
  }]

  obj._slowPeriods = {
    name: 'Slow SMA periods',
    defaultValue: 55
  }

  obj._fastPeriods = {
    name: 'Fast SMA periods',
    defaultValue: 21
  }

  obj._overColor = {
    name: 'Over color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }
  obj._overWickColor = {
    name: 'Over wick color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }
  obj._underColor = {
    name: 'Under color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }
  obj._underWickColor = {
    name: 'Under wick color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }

  obj.parameters = [
    obj._slowPeriods,
    obj._fastPeriods,
    obj._overColor,
    obj._overWickColor,
    obj._underColor,
    obj._underWickColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Bar
}

obj.exec = function (period) {
  // assert period1 > period2
  var prices = Plot.getTicks(period, obj._slowPeriods.value)
  if (prices == null) {
    return
  }

  var slowMa = Plot.sma(prices, obj._slowPeriods.value)
  var fastMa = Plot.sma(prices, obj._fastPeriods.value)

  var isSlowUnder = slowMa <= fastMa
  var ohlc = Plot.getOHLC(period)
  return [{
    value: ohlc,
    color: isSlowUnder ? obj._overColor.value : obj._underColor.value,
    wickColor: isSlowUnder ? obj._overWickColor.value : obj._underWickColor.value,
    hollow: ohlc.close >= ohlc.open
  }]
}
