var obj = {
  name: 'Elder Impulse System',
  group: 'Other',
}

obj.info = function () {
  obj.overlay = true
  obj.curves = [{
    title: 'Impulse System',
    type: Spock.CurveType.Bar,
    style: Spock.BarCurveStyle.Ohlc,
    hasPerBarColor: true,
    lineWidth: 2,
    zIndex: 1
  }]

  obj._emaPeriods = {
    name: 'EMA periods',
    defaultValue: 13
  }

  obj._fastPeriods = {
    name: 'MACD Fast periods',
    defaultValue: 12
  }

  obj._slowPeriods = {
    name: 'MACD Slow periods',
    defaultValue: 26
  }

  obj._signalPeriods = {
    name: 'MACD Signal periods',
    defaultValue: 9
  }

  obj._greenBarColor = {
    name: 'greenBar color',
    defaultValue: 'green',
    type: Spock.ParameterType.Color,
  }
  obj._greenBarWickColor = {
    name: 'greenBarWick color',
    defaultValue: 'green',
    type: Spock.ParameterType.Color,
  }
  obj._redBarColor = {
    name: 'redBar color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }
  obj._redBarWickColor = {
    name: 'redBarWick color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }
  obj._blueBarColor = {
    name: 'blueBar color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }
  obj._blueBarWickColor = {
    name: 'blueBarWick color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }

  obj.parameters = [
    obj._emaPeriods,
    obj._fastPeriods,
    obj._slowPeriods,
    obj._signalPeriods,
    obj._greenBarColor,
    obj._greenBarWickColor,
    obj._redBarColor,
    obj._redBarWickColor,
    obj._blueBarColor,
    obj._blueBarWickColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Bar
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._ema = new Array(obj._emaPeriods.value)
    obj._fast = new Array(obj._fastPeriods.value)
    obj._slow = new Array(obj._slowPeriods.value)
    obj._macd = new Array(obj._slowPeriods.value)
    obj._signal = new Array(obj._slowPeriods.value + obj._signalPeriods.value)
    obj._hist = new Array(obj._slowPeriods.value + obj._signalPeriods.value)
  }

  if (period === obj._emaPeriods.value) {
    obj._ema[period] = Plot.sma(Plot.getTicks(period, obj._emaPeriods.value), obj._emaPeriods.value)
  } else if (period > obj._emaPeriods.value) {
    obj._ema[period] = Plot.ema(Plot.getTick(period), obj._emaPeriods.value, obj._ema[period - 1])
  }

  if (period === obj._fastPeriods.value) {
    obj._fast[period] = Plot.sma(Plot.getTicks(period, obj._fastPeriods.value), obj._fastPeriods.value)
  } else if (period > obj._fastPeriods.value) {
    obj._fast[period] = Plot.ema(Plot.getTick(period), obj._fastPeriods.value, obj._fast[period - 1])
  }

  if (period === obj._slowPeriods.value) {
    obj._slow[period] = Plot.sma(Plot.getTicks(period, obj._slowPeriods.value), obj._slowPeriods.value)
  } else if (period > obj._slowPeriods.value) {
    obj._slow[period] = Plot.ema(Plot.getTick(period), obj._slowPeriods.value, obj._slow[period - 1])
  }

  if (period < obj._fastPeriods.value || period < obj._slowPeriods.value) {
    return
  }

  obj._macd[period] = obj._fast[period] - obj._slow[period]

  if (period === obj._slowPeriods.value + obj._signalPeriods.value) {
    obj._signal[period] = Plot.sma(obj._macd, obj._signalPeriods.value)
  } else if (period > obj._signalPeriods.value) {
    obj._signal[period] = Plot.ema(obj._macd[period], obj._signalPeriods.value, obj._signal[period - 1])
  }

  obj._hist[period] = obj._macd[period] !== undefined &&
    obj._signal[period] !== undefined ? obj._macd[period] - obj._signal[period] : null

  var isGreenBar = obj._ema[period] > obj._ema[period - 1] &&
    obj._hist[period] > obj._hist[period - 1]

  var isRedBar = !isGreenBar && obj._ema[period] < obj._ema[period - 1] &&
    obj._hist[period] < obj._hist[period - 1]

  var isBlueBar = !isGreenBar && !isRedBar

  var ohlc = Plot.getOHLC(period)
  return [{
    value: ohlc,
    color: isGreenBar ? obj._greenBarColor.value : isRedBar ? obj._redBarColor.value : obj._blueBarColor.value,
    wickColor: isGreenBar ? obj._greenBarWickColor.value : isRedBar ? obj._redBarWickColor.value : obj._blueBarWickColor.value,
    hollow: ohlc.close >= ohlc.open
  }]
}
