var obj = {
  name: 'HMA',
  summary: 'Hull Moving Average',
  description: 'https://alanhull.com/hull-moving-average',
}

obj.info = function () {
  obj.overlay = true;

  obj._curve = {
    title: 'HMA',
  }

  obj.curves = [obj._curve];

  obj._maPeriods = {
    name: 'Period',
    defaultValue: 20,
    // TODO(seamus): Add min value = 1, max value = 200, when supported in Latinum
  };

  obj.parameters = [obj._maPeriods];
  obj.requiredSource = Spock.RequiredSource.Tick;
};

function calc_wma (prices, wma_period) {
  // FIXME(seamus): Use more efficient way to calculate WMA
  // assert(prices.length == wma_period)
  var sum_weighted_prices = 0.;
  var sum_weights = 0;
  for (var i = 0, weight = 1, tot = wma_period; i < tot; i += 1, weight += 1) {
    sum_weighted_prices += prices[i] * weight;
    sum_weights += weight;
  }
  return sum_weighted_prices / sum_weights;
}

obj.exec = function (period) {
  var n = obj._maPeriods.value;
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._halfperiod = Math.floor(n / 2);
    obj._sqrtperiod = Math.floor(Math.sqrt(n));
    obj._madiff = new Array(n + obj._sqrtperiod);
  }

  if (period >= n) {
    var prices = Plot.getTicks(period, n);
    var ma1 = calc_wma(prices.slice(n - obj._halfperiod), obj._halfperiod) * 2;
    var ma2 = calc_wma(prices, n);
    obj._madiff[period] = ma1 - ma2;

    if (period >= n + obj._sqrtperiod - 1) {
      return [calc_wma(obj._madiff.slice(period - obj._sqrtperiod + 1), obj._sqrtperiod)];
    }
  }
  return null;
};
