var obj = {
  name: 'MaCloud',
  summary: 'Moving Average Cloud',
  description: `Draws two moving averages filling the area between them.
 MA type parameters can have a value of 0 for SMA or 1 for EMA.`,
}

obj.info = function () {
  obj.overlay = true

  obj._ma1Curve = {
    title: 'MA1',
    color: 'red',
  }

  obj._ma2Curve = {
    title: 'MA2',
    color: 'green',
  }

  obj.curves = [obj._ma1Curve, obj._ma2Curve, {
    title: 'Cloud',
    type: Spock.CurveType.Channel,
    color: '#46ffa500', // 70/255 alpha, orange
    showInLegend: false,
  }]

  obj._ma1MaPeriods = {
    name: 'MA1 periods',
    defaultValue: 20,
  }

  obj._ma1MaType = {
    name: 'MA1 MA type',
    defaultValue: 0,
  }

  obj._ma2MaPeriods = {
    name: 'MA2 periods',
    defaultValue: 21,
  }

  obj._ma2MaType = {
    name: 'MA2 MA type',
    defaultValue: 1,
  }

  obj.parameters = [obj._ma1MaPeriods, obj._ma1MaType, obj._ma2MaPeriods, obj._ma2MaType]
  obj.requiredSource = Spock.RequiredSource.Tick
}

const MaType = Object.freeze({ SMA: 0, EMA: 1 })

obj._buildCurveTitle = function (maType, maPeriods) {
  let maTypeStr;
  switch (maType) {
    case MaType.SMA:
      maTypeStr = 'SMA'
      break;
    case MaType.EMA:
      maTypeStr = 'EMA'
      break;
    default:
      break;
  }
  if (maTypeStr) {
    return `${maTypeStr}(${maPeriods})`
  }
  // TODO(seamus): Need a way to get list separator character for locale
  return `INVALID_MA_TYPE(${maType};${maPeriods})`
}

obj.preExec = function () {
  obj._ma1Curve.title = obj._buildCurveTitle(obj._ma1MaType.value, obj._ma1MaPeriods.value)
  obj._ma2Curve.title = obj._buildCurveTitle(obj._ma2MaType.value, obj._ma2MaPeriods.value)
}

obj.exec = function (period) {
  const maxLookback = Math.max(obj._ma1MaPeriods.value, obj._ma2MaPeriods.value)
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._ma1 = new Array(maxLookback)
    obj._ma2 = new Array(maxLookback)
  }

  if (period < maxLookback) {
    return
  }

  const prices = Plot.getTicks(period, maxLookback)
  if (!prices) {
    return
  }

  const firstValidMA1Period = maxLookback
  if (obj._ma1MaType.value == MaType.SMA) {
    if (period >= firstValidMA1Period)
      obj._ma1[period] = Plot.sma(prices, obj._ma1MaPeriods.value)
  } else if (obj._ma1MaType.value == MaType.EMA) {
    if (period === firstValidMA1Period)
      obj._ma1[period] = Plot.sma(prices, obj._ma1MaPeriods.value)
    else if (period > firstValidMA1Period)
      obj._ma1[period] = Plot.ema(prices[prices.length - 1], obj._ma1MaPeriods.value, obj._ma1[period - 1])
  }

  const firstValidMA2Period = maxLookback
  if (obj._ma2MaType.value == MaType.SMA) {
    if (period >= firstValidMA2Period)
      obj._ma2[period] = Plot.sma(prices, obj._ma2MaPeriods.value)
  } else if (obj._ma2MaType.value == MaType.EMA) {
    if (period === firstValidMA2Period)
      obj._ma2[period] = Plot.sma(prices, obj._ma2MaPeriods.value)
    else if (period > firstValidMA2Period)
      obj._ma2[period] = Plot.ema(prices[prices.length - 1], obj._ma2MaPeriods.value, obj._ma2[period - 1])
  }

  return [obj._ma1[period], obj._ma2[period], [obj._ma1[period], obj._ma2[period]]]
}
