var obj = {
  name: 'MA Multipliers',
  summary: 'Multiply MA by multiple factors',
  description: `Multiply MA by multiple factors.
 MA type parameter must have a value of 0 for SMA,
 other MA types are not supported.`
}

obj.info = function () {
  obj.overlay = true

  obj._ma1Curve = {
    title: 'Multiple 1',
    color: 'red',
    lineWidth: 2
  }

  obj._ma2Curve = {
    title: 'Multiple 2',
    color: 'green',
    lineWidth: 2
  }

  obj._ma3Curve = {
    title: 'Multiple 3',
    color: 'blue',
    lineWidth: 2
  }

  obj.curves = [obj._ma1Curve, obj._ma2Curve, obj._ma3Curve]

  obj._maPeriods = {
    name: 'MA periods',
    defaultValue: 70
  }

  obj._maType = {
    name: 'MA type',
    defaultValue: 0,
  }

  obj._mFactor1 = {
    name: 'Factor 1',
    defaultValue: 1.382,
    type: Spock.ParameterType.Double
  }

  obj._mFactor2 = {
    name: 'Factor 2',
    defaultValue: 1.618,
    type: Spock.ParameterType.Double
  }

  obj._mFactor3 = {
    name: 'Factor 3',
    defaultValue: 1.786,
    type: Spock.ParameterType.Double
  }

  obj.parameters = [obj._maPeriods, obj._maType, obj._mFactor1, obj._mFactor2, obj._mFactor3]
  obj.requiredSource = Spock.RequiredSource.Tick
}

const MaType = Object.freeze({ SMA: 0 })

// Copied from MaCloud.js
obj._buildCurveTitle = function (maType, maPeriods, factor) {
  let maTypeStr;
  switch (maType) {
    case MaType.SMA:
      maTypeStr = 'MA'
      break;
    default:
      break;
  }
  if (maTypeStr) {
    // FIXME(seamus): Localize
    return `${maTypeStr}(${maPeriods})*${factor}`
  }
  // TODO(seamus): Need a way to get list separator character for locale
  return `INVALID_MA_TYPE(${maType};${maPeriods})`
}

obj.preExec = function () {
  obj._ma1Curve.title = obj._buildCurveTitle(obj._maType.value, obj._maPeriods.value, obj._mFactor1.value)
  obj._ma2Curve.title = obj._buildCurveTitle(obj._maType.value, obj._maPeriods.value, obj._mFactor2.value)
  obj._ma3Curve.title = obj._buildCurveTitle(obj._maType.value, obj._maPeriods.value, obj._mFactor3.value)
}

obj.exec = function (period) {
  const n = obj._maPeriods.value
  if (period >= n - 1) {
    const smaV = Plot.sma(Plot.getTicks(period, n), n)
    return [obj._mFactor1.value * smaV, obj._mFactor2.value * smaV, obj._mFactor3.value * smaV]
  }
}
