var obj = {
  name: 'MACD',
  summary: 'Moving Average Convergence/Divergence'
}

obj.info = function () {
  obj._macdCurve = {
    title: 'MACD',
    lineWidth: 2
  }

  obj._signalCurve = {
    title: 'Signal',
    lineStyle: Spock.LineStyle.Dash,
    lineWidth: 2
  }

  obj._histogramCurve = {
    title: 'MACD Hist',
    style: Spock.LineCurveStyle.Histogram,
    lineWidth: 2,
    zIndex: -1,
    hasPerBarColor: true,
  }

  obj.curves = [obj._macdCurve, obj._signalCurve, obj._histogramCurve]

  obj._fastPeriods = {
    name: 'Fast periods',
    defaultValue: 12
  }

  obj._slowPeriods = {
    name: 'Slow periods',
    defaultValue: 26
  }

  obj._signalPeriods = {
    name: 'Signal periods',
    defaultValue: 9
  }

  obj._colorizeHistogram = {
    name: 'Colorize histogram',
    defaultValue: false,
    type: Spock.ParameterType.Bool,
    __domainLogic: false,
  }

  obj._aboveColor = {
    name: 'Above color',
    defaultValue: 'royalblue',
    type: Spock.ParameterType.Color,
  }

  obj._aboveFadeColor = {
    name: 'Above fade color',
    defaultValue: 'lightskyblue',
    type: Spock.ParameterType.Color,
  }

  obj._belowColor = {
    name: 'Below color',
    defaultValue: 'red',
    type: Spock.ParameterType.Color,
  }

  obj._belowFadeColor = {
    name: 'Below fade color',
    defaultValue: 'lightsalmon',
    type: Spock.ParameterType.Color,
  }

  obj.parameters = [
    obj._fastPeriods,
    obj._slowPeriods,
    obj._signalPeriods,
    obj._colorizeHistogram,
    obj._aboveColor,
    obj._aboveFadeColor,
    obj._belowColor,
    obj._belowFadeColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Tick
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._fast = new Array(obj._fastPeriods.value)
    obj._slow = new Array(obj._slowPeriods.value)
  }

  if (period === obj._fastPeriods.value) {
    obj._fast[period] = Plot.sma(Plot.getTicks(period, obj._fastPeriods.value), obj._fastPeriods.value)
  } else if (period > obj._fastPeriods.value) {
    obj._fast[period] = Plot.ema(Plot.getTick(period), obj._fastPeriods.value, obj._fast[period - 1])
  }

  if (period === obj._slowPeriods.value) {
    obj._slow[period] = Plot.sma(Plot.getTicks(period, obj._slowPeriods.value), obj._slowPeriods.value)
  } else if (period > obj._slowPeriods.value) {
    obj._slow[period] = Plot.ema(Plot.getTick(period), obj._slowPeriods.value, obj._slow[period - 1])
  }

  if (period < obj._fastPeriods.value || period < obj._slowPeriods.value) {
    return
  }

  var macd = obj._fast[period] - obj._slow[period]
  var signal_

  if (period === obj._slowPeriods.value + obj._signalPeriods.value) {
    // The MACD lines output hasn't been written yet,
    // so get a slice of the needed history excluding the current
    // period, then append the current periods MACD value
    var macdHistory = obj._macdCurve.output.slice(period - obj._signalPeriods.value, period - 1)
    macdHistory.push(macd)
    signal_ = Plot.sma(macdHistory, obj._signalPeriods.value)
  } else if (period > obj._signalPeriods.value) {
    signal_ = Plot.ema(macd, obj._signalPeriods.value, obj._signalCurve.output[period - 1])
  }

  var histogram = macd !== undefined && signal_ !== undefined ? macd - signal_ : null
  if (histogram !== null && obj._colorizeHistogram.value) {
    let color;
    if (period > obj._slowPeriods.value + obj._signalPeriods.value) {
      if (histogram >= 0) {
        let prevHisto = obj._histogramCurve.output[period - 1].value
        color = histogram > prevHisto ? obj._aboveColor.value : obj._aboveFadeColor.value
      } else if (histogram < 0) {
        let prevHisto = obj._histogramCurve.output[period - 1].value
        color = histogram < prevHisto ? obj._belowColor.value : obj._belowFadeColor.value
      }
    }

    return [macd, signal_, {
      value: histogram,
      color
    }]
  }
  return [macd, signal_, histogram]
}
