var obj = {
  name: 'TSI',
  summary: 'True Strength Index'
};

obj.info = function () {
  obj._tsiCurve = {
    title: 'TSI',
    lineWidth: 1
  };

  obj._signalCurve = {
    title: 'Signal',
    lineStyle: Spock.LineStyle.Dash,
    lineWidth: 1
  };

  obj.curves = [obj._tsiCurve, obj._signalCurve];

  obj._firstSmoothingPeriods = {
    name: 'First smoothing periods',
    defaultValue: 25
  };

  obj._secondSmoothingPeriods = {
    name: 'Second smoothing periods',
    defaultValue: 13
  };

  obj._signalPeriods = {
    name: 'Signal periods',
    defaultValue: 7
  };
  obj.parameters = [obj._firstSmoothingPeriods, obj._secondSmoothingPeriods, obj._signalPeriods];
  obj.requiredSource = Spock.RequiredSource.Tick;
};

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._priceChange = [];
    obj._priceChangeSmoothed = [];
    obj._priceChangeDoubleSmoothed = [];
    obj._absPriceChange = [];
    obj._absPriceChangeSmoothed = [];
    obj._absPriceChangeDoubleSmoothed = [];
    obj._tsi = [];
    obj._signal = [];
  }

  if (period < 1) {
    return;
  }

  var prices = Plot.getTicks(period, 2);
  if (prices == null) {
    return;
  }
  var currentPrice = prices[1];
  var priorPrice = prices[0];
  var priceChange = currentPrice - priorPrice;
  var absPriceChange = Math.abs(priceChange);
  obj._priceChange[period] = priceChange;
  obj._absPriceChange[period] = absPriceChange;

  // plus 1 since prior price is required
  var firstSmoothedPeriod = obj._firstSmoothingPeriods.value + 1;
  var firstDoubleSmoothedPeriod = obj._firstSmoothingPeriods.value + obj._secondSmoothingPeriods.value;

  if (period === firstSmoothedPeriod) {
    obj._priceChangeSmoothed[period] = Plot.sma(obj._priceChange, obj._firstSmoothingPeriods.value);
    obj._absPriceChangeSmoothed[period] = Plot.sma(obj._absPriceChange, obj._firstSmoothingPeriods.value);
  } else if (period > firstSmoothedPeriod) {
    obj._priceChangeSmoothed[period] = Plot.ema(priceChange, obj._firstSmoothingPeriods.value, obj._priceChangeSmoothed[period - 1]);
    obj._absPriceChangeSmoothed[period] = Plot.ema(absPriceChange, obj._firstSmoothingPeriods.value, obj._absPriceChangeSmoothed[period - 1]);
  }

  if (period === firstDoubleSmoothedPeriod) {
    obj._priceChangeDoubleSmoothed[period] = Plot.sma(obj._priceChangeSmoothed, obj._secondSmoothingPeriods.value);
    obj._absPriceChangeDoubleSmoothed[period] = Plot.sma(obj._absPriceChangeSmoothed, obj._secondSmoothingPeriods.value);
  } else if (period > firstDoubleSmoothedPeriod) {
    obj._priceChangeDoubleSmoothed[period] = Plot.ema(obj._priceChangeSmoothed[period], obj._secondSmoothingPeriods.value, obj._priceChangeDoubleSmoothed[period - 1]);
    obj._absPriceChangeDoubleSmoothed[period] = Plot.ema(obj._absPriceChangeSmoothed[period], obj._secondSmoothingPeriods.value, obj._absPriceChangeDoubleSmoothed[period - 1]);
  }

  if (period < firstSmoothedPeriod || period < firstDoubleSmoothedPeriod) {
    return;
  }

  var tsi = 100 * (obj._priceChangeDoubleSmoothed[period] / obj._absPriceChangeDoubleSmoothed[period]);
  obj._tsi[period] = period;
  var signal_;

  var firstSignalPeriod = firstDoubleSmoothedPeriod + obj._signalPeriods.value;
  if (period === firstSignalPeriod) {
    signal_ = Plot.sma(obj._tsi, obj._signalPeriods.value);
    obj._signal[period] = signal_;
  } else if (period > firstSignalPeriod) {
    signal_ = Plot.ema(tsi, obj._signalPeriods.value, obj._signal[period - 1]);
    obj._signal[period] = signal_;
  }

  return [tsi, signal_];
};
