var obj = {
  name: 'VolumeEma',
  summary: 'Volume with EMA'
}

obj.info = function () {
  obj._maCurve = {
    title: 'EMA',
    lineStyle: Spock.LineStyle.Dash,
    color: 'darkorange',
    lineWidth: 2
  }

  obj.curves = [{
    title: 'Volume',
    style: Spock.LineCurveStyle.Columns,
    lineWidth: 1,
    hasPerBarColor: true
  }, obj._maCurve]

  obj._maPeriods = {
    name: 'EMA periods',
    defaultValue: 14
  }
  obj._colorBasedOnPreviousClose = {
    name: 'Color based on previous close',
    defaultValue: false,
    type: Spock.ParameterType.Bool,
    __domainLogic: false,
  }

  obj._buyingColor = {
    name: 'Buying color',
    defaultValue: 'forestgreen',
    type: Spock.ParameterType.Color,
  }

  obj._evenColor = {
    name: 'Even color',
    defaultValue: 'gray',
    type: Spock.ParameterType.Color,
  }

  obj._sellingColor = {
    name: 'Selling color',
    defaultValue: 'crimson',
    type: Spock.ParameterType.Color,
  }

  obj._maCurve.__associatedParameters = [obj._maPeriods]

  obj.parameters = [
    obj._maPeriods,
    obj._colorBasedOnPreviousClose,
    obj._buyingColor,
    obj._evenColor,
    obj._sellingColor,
  ]
  obj.requiredSource = Spock.RequiredSource.Bar
}

function qFuzzyCompare (p1, p2) {
  return (Math.abs(p1 - p2) * 1000000000000.0 <= Math.min(Math.abs(p1), Math.abs(p2)));
}

obj.exec = function (period) {
  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._ema = new Array(obj._maPeriods.value);
  }

  var vol;
  if (period === obj._maPeriods.value) {
    var vols = Plot.volume(period, obj._maPeriods.value);
    vol = vols[vols.length - 1];
    obj._ema[period] = Plot.sma(vols, obj._maPeriods.value);
  } else if (period > obj._maPeriods.value) {
    vol = Plot.volume(period);
    obj._ema[period] = Plot.ema(vol, obj._maPeriods.value, obj._ema[period - 1]);
  } else {
    vol = Plot.volume(period);
  }

  var color;
  if (period > 0) {
    var newPrice = Plot.close(period);
    var oldPrice = obj._colorBasedOnPreviousClose.value ? Plot.close(period - 1) : Plot.open(period);
    color = qFuzzyCompare(newPrice, oldPrice) ? obj._evenColor.value : newPrice > oldPrice ? obj._buyingColor.value : obj._sellingColor.value;
  } else {
    color = obj._evenColor.value;
  }

  return [{
    value: vol,
    color: color
  }, obj._ema[period]];
}
