var obj = {
  name: 'VPT',
  summary: 'Volume Price Trend with Signal Line',
  description: `MA type parameters can have a value of 0 for SMA or 1 for EMA.
 The Signal Line is applied to the unsmoothed VPT.
 Use a VPT smoothing period of 1 to disable smoothing.`,
}

obj.info = function () {
  obj.curves = [{
    title: 'VPT',
    color: 'deepskyblue',
    lineWidth: 2,
  }, {
    title: 'Signal',
    color: 'red',
    lineWidth: 2,
  }]

  obj._vptSmoothingPeriods = {
    name: 'VPT smoothing periods',
    defaultValue: 1,
  }

  obj._vptSmoothingMaType = {
    name: 'VPT smoothing MA type',
    defaultValue: 0,
  }

  obj._signalMaPeriods = {
    name: 'Signal periods',
    defaultValue: 30,
  }

  obj._signalMaType = {
    name: 'Signal MA type',
    defaultValue: 1,
  }

  obj.parameters = [obj._vptSmoothingPeriods, obj._vptSmoothingMaType, obj._signalMaPeriods, obj._signalMaType]
  obj.requiredSource = Spock.RequiredSource.Tick
}

const MaType = Object.freeze({ SMA: 0, EMA: 1 })

obj.exec = function (period) {
  const firstSmoothedVptPeriod = obj._vptSmoothingPeriods.value
  const firstMaPeriod = obj._signalMaPeriods.value

  if (Plot.barUpdateMode === Spock.BarUpdateMode.AllBars) {
    obj._vpt = [0]
    obj._smoothedVpt = new Array(firstSmoothedVptPeriod)
    obj._sig = new Array(firstMaPeriod)
  }

  if (period < 1)
    return;

  const prices = Plot.getTicks(period, 2)
  obj._vpt[period] = obj._vpt[period - 1] + Plot.volume(period) * ((prices[1] - prices[0]) / prices[0])

  if (obj._vptSmoothingMaType.value == MaType.SMA) {
    if (period >= firstSmoothedVptPeriod)
      obj._smoothedVpt[period] = Plot.sma(obj._vpt, obj._vptSmoothingPeriods.value)
  } else if (obj._vptSmoothingMaType.value == MaType.EMA) {
    if (period === firstSmoothedVptPeriod)
      obj._smoothedVpt[period] = Plot.sma(obj._vpt, obj._vptSmoothingPeriods.value)
    else if (period > firstSmoothedVptPeriod)
      obj._smoothedVpt[period] = Plot.ema(obj._vpt[period], obj._vptSmoothingPeriods.value, obj._smoothedVpt[period - 1])
  }

  if (obj._signalMaType.value == MaType.SMA) {
    if (period >= firstMaPeriod)
      obj._sig[period] = Plot.sma(obj._vpt, obj._signalMaPeriods.value)
  } else if (obj._signalMaType.value == MaType.EMA) {
    if (period === firstMaPeriod)
      obj._sig[period] = Plot.sma(obj._vpt, obj._signalMaPeriods.value)
    else if (period > firstMaPeriod)
      obj._sig[period] = Plot.ema(obj._vpt[period], obj._signalMaPeriods.value, obj._sig[period - 1])
  }

  return [obj._smoothedVpt[period], obj._sig[period]]
}

